#    Megha uses the standard Charissa Experiment Control with these changes

set VERSION "Megha"

loadsource sys.common

set frametitle "MIDAS Data Acquisition Experiment Control (MEGHA)"

set CAMACPresent 1
set CAMACEnable 1

set TSEnable 1

set txEnable 1

set histEnable 1
set HistMode 3

set SetupMessage "\n\
Are you sure you wish to SETUP the data acquisition system?\n\n\
SETUP will STOP data acquisition.\n\
Configure the VME Control software and restore VME parameter settings.\n\
It will also recreate all histograms selected.\n\
This may take several minutes.\n\n\
If you decide to continue please wait for the SETUP sequence to complete and DO NOT touch the system while it is in progress.
"

set TestBuffer "ffff0008 20428100 20438180 20868200 20878280 ffff000c 204e8300 204f8380 208e8400 208f8480 208c8500 208d8580 20908400 20918480 20928500 20938580"

if {[info exists env(CamacController)]} then {set resource $env(CamacController)}  \
   else {set resource RS232CC}
set CAMACcontroller $resource

#    number of CAMAC crates
set NumCrates 4

set Maxmodule 2

lappend actionlist [list "Show histogram name translations" "Show_Xlate"]
lappend actionlist [list "Edit histogram translation file"  "edit_xlate"]
lappend actionlist [list "Setup histogramming"  "Setup_histograms"]

proc setup_histograms {} {
   global SpectrumServer 

   set maxmodule [inform camaccontrol inquire lc2277count]
   set range [expr ($maxmodule+1)*64]

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   if {$z == 0} then {map_spectrum "Stat 0 0"}

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]
}

proc Setup_histograms {} {
    global RDOUThdwre

#    create and enable histograms

     set-footer "Setting up histogramming"
     set-busy
     unmap_spectra
     delete_spectra
     if {$RDOUThdwre != "VME"} then {ClearConfig}
     create_spectra
     clear-busy
     clear-footer
}

proc SpectrumName {n} {Give_Xlate $n}

proc create_spectra {} {
   global SpectrumServer HistMode EGreport Maxmodule

   set maxmodule [inform camaccontrol inquire lc2277count]
   set range [expr ($maxmodule+1)*64]

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat 0 0"

   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"
   set-busy

   set EGreport_SAVE $EGreport
   set EGreport -1

   Make_Xlate $SpectrumServer

   set i 2

   for {set m 1} {$m <= $Maxmodule} {incr m} {
   for {set c 0} {$c <= 31} {incr c} {

   set-footer "m=$m c=$c"

    if {$HistMode == 1 || $HistMode == 3} then {

      set spectrumname [SpectrumName m#$m.c$c.E]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 8192 :name '$spectrumname' !new)" } r]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $r==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $r "Attempting to create spectrum $spectrumname"
          clear-footer
          error $r
      } else {
          incr i
          map_spectrum  "$spectrumname $m [expr ($c * 2) + 1]"
      }
    }

    if {$HistMode == 2 || $HistMode == 3} then {

      set spectrumname [SpectrumName m#$m.c$c.T]

      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 8192 :name '$spectrumname' !new)" } r]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $r==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $r "Attempting to create spectrum $spectrumname"
          clear-footer
          error $r
      } else {
          incr i
          map_spectrum  "$spectrumname $m [expr $c * 2]"
      }
    }

   }
   }

   set EGreport $EGreport_SAVE

   clear-busy
   set-footer "$i spectra created"
}

proc click-camac {action} {
  global CAMACEnable CAMACserver CAMACcontroller EGallow
  global NumCrates

  if {$CAMACEnable == 0} then {return}

   starttask camaccontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
    }

   set-busy

   if {$action == 2} then {
     for {set c 1} {$c <= $NumCrates} {incr c} {inform RS232CC initialise $c}
   }

   do_inhibit_camac
 
   set-footer "sending $p to CAMAC control task"
   set z [catch {send camaccontrol $p} m]

   if {$action == 1} then {do_uninhibit_camac}

   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}
   redisplay-camac
}

proc edit_xlate {}  {
   global XlatePres Xlate
   global env SpectrumServer

   set maxmodule [inform camaccontrol inquire lc2277count]

   if {$XlatePres == 0} then {Make_Xlate $SpectrumServer}

   set contents ""

   for {set m 1} {$m <= $maxmodule} {incr m} {
      for {set c 1} {$c <= 32} {incr c} {

         set s ($m.$c).E
         if {$s != [SpectrumName $s]} then {
             append contents [format "%24s %24s\n" $s $Xlate($s)]
         } else {
             append contents [format "%24s %24s\n" $s ""]
         }

         set s ($m.$c).T
         if {$s != [SpectrumName $s]} then {
             append contents [format "%24s %24s\n" $s $Xlate($s)]
         } else {
             append contents [format "%24s %24s\n" $s ""]
         }
      }
   }

    set contents [split [Edit_Xlate $contents] \n]

    if {[llength $contents] == 0} then {return}

#    data has been modified  - save to file

    set file [file join $env(EXPERIMENTBASE) Histconfigs [set SpectrumServer]_names]
    set z [catch {open $file w} f]
    if {$z != 0} then {midas-warning "Unable to write file $file\n$f"; return}

    foreach item $contents {
       if {[llength $item] == 2} then {
           puts $f "$item"
       }
    }
    close $f
    set-footer "$file updated"

#  update the arrays

    Make_Xlate $SpectrumServer
}
